# coding=utf-8
from __future__ import print_function

import argparse
import os
import re

import cv2
import numpy as np

from suanpan import path, utils
from suanpan.arguments import *  # pylint: disable-msg=w0614
from suanpan.components import Result
from suanpan.datawarehouse import dw
from suanpan.io import storage


class Table(Arg):
    def __init__(self, key, table, partition):
        super(HiveTable, self).__init__(key)
        self.table = String(key=table, required=True)
        self.partition = String(key=partition)

    @property
    def isSet(self):
        return True

    def addParserArguments(self, parser):
        self.table.addParserArguments(parser)
        self.partition.addParserArguments(parser)

    def load(self, args):
        self.table.load(args)
        self.partition.load(args)
        self.value = dict(table=self.table.value, partition=self.partition.value)

    def format(self, context):
        self.value = dw.readTable(self.table.value, self.partition.value)
        return self.value

    def save(self, context, result):
        data = result.value
        dw.writeTable(self.table.value, data)
        return self.value


HiveTable = Table


class SeriesRules(ListOfString):
    OPERATORS = {
        ">": np.greater,
        ">=": np.greater_equal,
        "<": np.less,
        "<=": np.less_equal,
    }

    @classmethod
    def transform(cls, item):
        key, operate, value = re.match(r"^(\w+)\s*([<>]=?)\s*(\d+)$", item).groups()
        return {"key": key, "operate": cls.getOperator(operate), "value": float(value)}

    @classmethod
    def getOperator(cls, name):
        return cls.OPERATORS[name]


class LogicalOperator(String):
    LOGICAL_OPERATORS = {
        "and": np.logical_and,
        "or": np.logical_or,
        "xor": np.logical_xor,
        "not": np.logical_not,
    }

    def format(self, context):
        try:
            self.value = self.getLogicalOperator(self.value)
            return self.value
        except:
            argparse.ArgumentTypeError(
                "Unknown logical operator: {}, should in {}".format(
                    self.value, self.LOGICAL_OPERATORS
                )
            )

    @classmethod
    def getLogicalOperator(cls, name):
        return cls.LOGICAL_OPERATORS[name]


class File(String):
    FILENAME = "file"
    FILETYPE = None

    def __init__(self, key, **kwargs):
        fileName = kwargs.pop("name", self.FILENAME)
        fileType = kwargs.pop("type", self.FILETYPE)
        self.fileName = (
            "{}.{}".format(fileName, fileType.lower()) if fileType else fileName
        )
        super(File, self).__init__(key, **kwargs)

    def load(self, args):
        self.objectPrefix = super(File, self).load(args)
        self.objectName = (
            storage.pathJoin(self.objectPrefix, self.fileName)
            if self.objectPrefix
            else None
        )
        self.filePath = (
            storage.getPathInTempStore(self.objectName) if self.objectName else None
        )
        if self.filePath:
            path.safeMkdirsForFile(self.filePath)
        self.value = self.filePath
        return self.filePath

    def format(self, context):
        if self.filePath:
            storage.download(self.objectName, self.filePath)
        return self.filePath

    def save(self, context, result):
        filePath = result.value
        storage.upload(self.objectName, filePath)
        return self.objectPrefix


class Folder(String):
    def load(self, args):
        self.folderName = super(Folder, self).load(args)
        self.folderPath = (
            storage.getPathInTempStore(self.folderName) if self.folderName else None
        )
        if self.folderPath:
            path.safeMkdirs(self.folderPath)
        self.value = self.folderPath
        return self.folderPath

    def format(self, context):
        if self.folderPath:
            storage.download(self.folderName, self.folderPath)
        return self.folderPath

    def clean(self, context):
        if self.folderPath:
            path.empty(self.folderPath)
        return self.folderPath

    def save(self, context, result):
        folderPath = result.value
        storage.upload(self.folderName, folderPath)
        return self.folderName


class Data(File):
    FILENAME = "data"


class Csv(Data):
    FILETYPE = "csv"

    def format(self, context):
        super(Csv, self).format(context)
        self.value = utils.loadFromCsv(self.filePath)
        return self.value

    def save(self, context, result):
        utils.saveAsCsv(self.filePath, result.value)
        return super(Csv, self).save(context, Result(self.filePath))


class Npy(Data):
    FILETYPE = "npy"

    def format(self, context):
        super(Npy, self).format(context)
        self.value = utils.loadFromNpy(self.filePath)
        return self.value

    def save(self, context, result):
        utils.saveAsNpy(self.filePath, result.value)
        return super(Npy, self).save(context, Result(self.filePath))


class Model(File):
    FILENAME = "model"


class H5Model(Model):
    FILETYPE = "h5"


class Checkpoint(Model):
    FILETYPE = "ckpt"


class JsonModel(Model):
    FILETYPE = "json"


class Image(File):
    FILENAME = "image"
    FILETYPE = "png"

    def format(self, context):
        filepath = super(Image, self).format(context)
        self.value = cv2.imread(filepath)  # pylint: disable-msg=e1101
        return self.value

    def save(self, context, result):
        img = result.value
        filepath = storage.getPathInTempStore(self.objectName)
        path.safeMkdirsForFile(filepath)
        cv2.imwrite(filepath, img)  # pylint: disable-msg=e1101
        return super(Image, self).save(context, Result(self.filePath))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from suanpan.services import common_pb2 as common__pb2


class CommonStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
        """Constructor.

    Args:
      channel: A grpc.Channel.
    """
        self.predict = channel.unary_unary(
            "/common.Common/predict",
            request_serializer=common__pb2.Request.SerializeToString,
            response_deserializer=common__pb2.Response.FromString,  # pylint: disable=e1101
        )
        self.predictAsync = channel.unary_unary(
            "/common.Common/predictAsync",
            request_serializer=common__pb2.Request.SerializeToString,
            response_deserializer=common__pb2.Response.FromString,  # pylint: disable=e1101
        )


class CommonServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def predict(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def predictAsync(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CommonServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "predict": grpc.unary_unary_rpc_method_handler(
            servicer.predict,
            request_deserializer=common__pb2.Request.FromString,  # pylint: disable=e1101
            response_serializer=common__pb2.Response.SerializeToString,
        ),
        "predictAsync": grpc.unary_unary_rpc_method_handler(
            servicer.predictAsync,
            request_deserializer=common__pb2.Request.FromString,  # pylint: disable=e1101
            response_serializer=common__pb2.Response.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "common.Common", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))



# coding=utf-8
from __future__ import print_function

from suanpan.io.oss import OssStorage
from suanpan.proxy import Proxy


class StorageProxy(Proxy):
    MAPPING = {"oss": OssStorage}


storage = StorageProxy()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common.proto

import sys

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="common.proto",
    package="common",
    syntax="proto3",
    serialized_options=_b(
        "\n\037com.shuzhi.pmmlgrpc.pmml.commonB\013CommonProtoP\001\242\002\002CM"
    ),
    serialized_pb=_b(
        '\n\x0c\x63ommon.proto\x12\x06\x63ommon"s\n\x07Request\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0b\n\x03in1\x18\x03 \x01(\t\x12\x0b\n\x03in2\x18\x04 \x01(\t\x12\x0b\n\x03in3\x18\x05 \x01(\t\x12\x0b\n\x03in4\x18\x06 \x01(\t\x12\x0b\n\x03in5\x18\x07 \x01(\t\x12\r\n\x05\x65xtra\x18\x08 \x01(\t"\x82\x01\n\x08Response\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0f\n\x07success\x18\x02 \x01(\x08\x12\x0b\n\x03msg\x18\x03 \x01(\t\x12\x0c\n\x04out1\x18\x04 \x01(\t\x12\x0c\n\x04out2\x18\x05 \x01(\t\x12\x0c\n\x04out3\x18\x06 \x01(\t\x12\x0c\n\x04out4\x18\x07 \x01(\t\x12\x0c\n\x04out5\x18\x08 \x01(\t2m\n\x06\x43ommon\x12.\n\x07predict\x12\x0f.common.Request\x1a\x10.common.Response"\x00\x12\x33\n\x0cpredictAsync\x12\x0f.common.Request\x1a\x10.common.Response"\x00\x42\x35\n\x1f\x63om.shuzhi.pmmlgrpc.pmml.commonB\x0b\x43ommonProtoP\x01\xa2\x02\x02\x43Mb\x06proto3'
    ),
)


_REQUEST = _descriptor.Descriptor(
    name="Request",
    full_name="common.Request",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="id",
            full_name="common.Request.id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="type",
            full_name="common.Request.type",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="in1",
            full_name="common.Request.in1",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="in2",
            full_name="common.Request.in2",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="in3",
            full_name="common.Request.in3",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="in4",
            full_name="common.Request.in4",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="in5",
            full_name="common.Request.in5",
            index=6,
            number=7,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="extra",
            full_name="common.Request.extra",
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=24,
    serialized_end=139,
)


_RESPONSE = _descriptor.Descriptor(
    name="Response",
    full_name="common.Response",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="request_id",
            full_name="common.Response.request_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="success",
            full_name="common.Response.success",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="msg",
            full_name="common.Response.msg",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="out1",
            full_name="common.Response.out1",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="out2",
            full_name="common.Response.out2",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="out3",
            full_name="common.Response.out3",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="out4",
            full_name="common.Response.out4",
            index=6,
            number=7,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="out5",
            full_name="common.Response.out5",
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=142,
    serialized_end=272,
)

DESCRIPTOR.message_types_by_name["Request"] = _REQUEST
DESCRIPTOR.message_types_by_name["Response"] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType(
    "Request",
    (_message.Message,),
    dict(
        DESCRIPTOR=_REQUEST,
        __module__="common_pb2"
        # @@protoc_insertion_point(class_scope:common.Request)
    ),
)
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType(
    "Response",
    (_message.Message,),
    dict(
        DESCRIPTOR=_RESPONSE,
        __module__="common_pb2"
        # @@protoc_insertion_point(class_scope:common.Response)
    ),
)
_sym_db.RegisterMessage(Response)


DESCRIPTOR._options = None

_COMMON = _descriptor.ServiceDescriptor(
    name="Common",
    full_name="common.Common",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    serialized_start=274,
    serialized_end=383,
    methods=[
        _descriptor.MethodDescriptor(
            name="predict",
            full_name="common.Common.predict",
            index=0,
            containing_service=None,
            input_type=_REQUEST,
            output_type=_RESPONSE,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name="predictAsync",
            full_name="common.Common.predictAsync",
            index=1,
            containing_service=None,
            input_type=_REQUEST,
            output_type=_RESPONSE,
            serialized_options=None,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_COMMON)

DESCRIPTOR.services_by_name["Common"] = _COMMON

# @@protoc_insertion_point(module_scope)



# coding=utf-8
from __future__ import print_function

import tempfile

from suanpan.docker import DockerComponent
from suanpan.arguments import String, Int


class ToolComponent(DockerComponent):
    dataWarehouseArguments = [
        String("dw-type", default="hive"),
        String("dw-hive-host", default="47.94.82.175"),
        Int("dw-hive-port", default=10000),
        String("dw-hive-database", default="default"),
        String("dw-hive-username", default="spark"),
        String("dw-hive-password"),
        String("dw-hive-auth"),
        String("dw-odps-access-id"),
        String("dw-odps-access-key"),
        String(
            "dw-odps-endpoint", default="http://service.cn.maxcompute.aliyun.com/api"
        ),
        String("dw-odps-project"),
    ]
    storageArguments = [
        String("storage-type", default="oss"),
        String("storage-oss-access-id", default="LTAIgV6cMz4TgHZB"),
        String("storage-oss-access-key", default="M6jP8a1KN2kfZR51M08UiEufnzQuiY"),
        String("storage-oss-bucket-name", default="suanpan"),
        String("storage-oss-endpoint", default="http://oss-cn-beijing.aliyuncs.com"),
        String("storage-oss-delimiter", default="/"),
        String("storage-oss-temp-store", default=tempfile.gettempdir()),
        Int("storage-oss-download-num-threads", default=1),
        String("storage-oss-download-store-name", default=".py-oss-download"),
        Int("storage-oss-upload-num-threads", default=1),
        String("storage-oss-upload-store-name", default=".py-oss-upload"),
    ]
    defaultArguments = dataWarehouseArguments + storageArguments



# coding=utf-8
from __future__ import print_function

import collections
import functools
import json

import cv2
import imageio
import numpy as np
import pandas as pd

from suanpan import asyncio, convert, path
from suanpan.log import logger


def loadFromCsv(filepath, **kwargs):
    kwargs.setdefault("index_col", 0)
    return pd.read_csv(filepath, **kwargs)


def saveAsCsv(filepath, data, **kwargs):
    path.safeMkdirsForFile(filepath)
    data.to_csv(filepath, **kwargs)
    return filepath


def loadFromNpy(filepath, **kwargs):
    kwargs.setdefault("encoding", "latin1")
    return np.load(filepath, **kwargs)


def saveAsNpy(filepath, data, **kwargs):
    path.safeMkdirsForFile(filepath)
    kwargs.setdefault("fix_imports", True)
    np.save(filepath, data, **kwargs)
    return filepath


def loadFromJson(filepath, **kwargs):
    encoding = kwargs.pop("encoding", "utf-8")
    with open(filepath, "r", encoding=encoding) as file:
        return json.load(file, **kwargs)


def saveAsJson(filepath, data, **kwargs):
    path.safeMkdirsForFile(filepath)
    encoding = kwargs.pop("encoding", "utf-8")
    with open(filepath, "w", encoding=encoding) as file:
        json.dump(data, file, **kwargs)
    return filepath


def saveImage(filepath, image):
    path.safeMkdirsForFile(filepath)
    cv2.imwrite(filepath, image)  # pylint: disable-msg=e1101
    return filepath


def saveAsFlatImage(filepath, data):
    image = convert.flatAsImage(data)
    return saveImage(filepath, image)


def saveAsAnimatedGif(filepath, data):
    image3D = convert.to3D(data)
    path.safeMkdirsForFile(filepath)
    imageio.mimsave(filepath, image3D)
    return filepath


def saveAsImage(filepath, data, flag=None):
    mapping = {None: saveImage, "flat": saveAsFlatImage, "animated": saveAsAnimatedGif}
    func = mapping.get(flag)
    if not func:
        raise Exception("Unknow flag: {}".format(flag))
    return func(filepath, data)


def saveAsImages(filepathPrefix, images, workers=None):
    counts = len(images)
    n = len(str(counts))
    workers = workers or min(counts, asyncio.WORKERS)
    with asyncio.multiThread(workers) as pool:
        for index, image in enumerate(images):
            pool.apply_async(
                saveAsImage,
                args=("{}_{}.png".format(filepathPrefix, str(index).zfill(n)), image),
            )
    return filepathPrefix


def saveAllAsImages(filepathPrefix, data, workers=None, pool=None):
    def _save(filepathPrefix, image3D, pool):
        layers = len(image3D)
        n = len(str(layers))
        for index, image in enumerate(image3D):
            pool.apply_async(
                saveImage,
                args=("{}_{}.png".format(filepathPrefix, str(index).zfill(n)), image),
            )
        pool.apply_async(
            saveAsFlatImage, args=("{}.png".format(filepathPrefix), image3D)
        )
        pool.apply_async(
            saveAsAnimatedGif, args=("{}.gif".format(filepathPrefix), image3D)
        )

    image3D = convert.to3D(data)
    if pool:
        _save(filepathPrefix, image3D, pool)
    else:
        layers = len(image3D)
        workers = workers or min(layers + 2, asyncio.WORKERS)
        with asyncio.multiThread(workers) as pool:
            _save(filepathPrefix, image3D, pool)
    return filepathPrefix


def merge(*dicts):
    def _merge(result, item):
        result.update(item)
        return result

    return functools.reduce(_merge, dicts, {})


def count(iterable):
    if hasattr(iterable, "__len__"):
        return len(iterable)

    d = collections.deque(enumerate(iterable, 1), maxlen=1)
    return d[0][0] if d else 0



# coding=utf-8
from __future__ import print_function

import tempfile
import traceback
from contextlib import contextmanager

from pyhive import hive

from suanpan import Context
from suanpan.arguments import Int, String
from suanpan.components import Component
from suanpan.datawarehouse import dw
from suanpan.io import storage
from suanpan.log import logger


class DockerComponent(Component):

    hiveArguments = [
        String("dw-hive-host", default="localhost"),
        Int("dw-hive-port"),
        String("dw-hive-database", default="default"),
        String("dw-hive-username"),
        String("dw-hive-password"),
        String("dw-hive-auth"),
    ]
    odpsArguments = [
        String("dw-odps-access-id"),
        String("dw-odps-access-key"),
        String(
            "dw-odps-endpoint", default="http://service.cn.maxcompute.aliyun.com/api"
        ),
        String("dw-odps-project"),
    ]

    dataWarehouseBaseArguments = [String("dw-type", default="hive")]
    dataWarehouseArguments = dataWarehouseBaseArguments + hiveArguments + odpsArguments
    storageArguments = [
        String("storage-type", default="oss"),
        String("storage-oss-access-id"),
        String("storage-oss-access-key"),
        String("storage-oss-bucket-name", default="suanpan"),
        String("storage-oss-endpoint", default="http://oss-cn-beijing.aliyuncs.com"),
        String("storage-oss-delimiter", default="/"),
        String("storage-oss-temp-store", default=tempfile.gettempdir()),
        Int("storage-oss-download-num-threads", default=1),
        String("storage-oss-download-store-name", default=".py-oss-download"),
        Int("storage-oss-upload-num-threads", default=1),
        String("storage-oss-upload-store-name", default=".py-oss-upload"),
    ]
    defaultArguments = dataWarehouseArguments + storageArguments

    @contextmanager
    def context(self, args):
        self.storage = self.setStorage(args)
        self.dw = self.setDataWarehouse(args)
        yield Context(dw=self.dw, storage=self.storage)

    def setDataWarehouse(self, args):
        return dw.setBackend(
            **self.defaultArgumentsFormat(args, self.dataWarehouseArguments)
        )

    def setStorage(self, args):
        return storage.setBackend(
            **self.defaultArgumentsFormat(args, self.storageArguments)
        )

    def defaultArgumentsFormat(self, args, arguments):
        arguments = (arg.key.replace("-", "_") for arg in arguments)
        return {
            self.defaultArgumentKeyFormat(arg): getattr(args, arg) for arg in arguments
        }

    def defaultArgumentKeyFormat(self, key):
        return self.toCamelCase(self.removePrefix(key))

    def removePrefix(self, string, delimiter="_", num=1):
        pieces = string.split(delimiter)
        pieces = pieces[num:] if len(pieces) > num else pieces
        return delimiter.join(pieces)

    def toCamelCase(self, string, delimiter="_"):
        camelCaseUpper = lambda i, s: s[0].upper() + s[1:] if i and s else s
        return "".join(
            [camelCaseUpper(i, s) for i, s in enumerate(string.split(delimiter))]
        )



# coding=utf-8
from __future__ import print_function

from suanpan.datawarehouse.hive import HiveDataWarehouse
from suanpan.datawarehouse.odps import OdpsDataWarehouse
from suanpan.proxy import Proxy


class DataWarehouseProxy(Proxy):
    MAPPING = {"hive": HiveDataWarehouse, "odps": OdpsDataWarehouse}


dw = DataWarehouseProxy()
